# STORER.RL
# This module has two functions
# 1. It stores new data directly, if user wants to add new case obtained
#    manually
# 2. It stores the emerging solution in the case base if it is found
#    worth for future problem solving

TITLE  Case Storing Module

$INCLUDE global.dcl
$INCLUDE storer.dcl

GOAL     new solution is stored

#
RULE     case card file 1
IF       case index IS type1
THEN     case card file IS type1.cdi

#
RULE     case card file 2
IF       case index IS type2
THEN     case card file IS type2.cdi

#
RULE     case card file 3
IF       case index IS type3_4
THEN     case card file IS type3_4.cdi

#
RULE     case card file 4
IF       case index IS type3_5
THEN     case card file IS type3_4.cdi

# First try to load case card. If not available, create case card.
#
RULE     case card creation 1
IF       CBR_load_case_card(case index) == 1
THEN     case card created

#
RULE     case card creation 2
IF       CBR_create_case_card(case card file) == 1
THEN     case card created

#
RULE     writing case information to data file 1
IF       WRITE cinfo.dat NUMERIC: 0 STRING: solution name
AND      WRITE cinfo.dat NUMERIC: 3 STRING: FRAME(UserRequirements, LivingDiningRequirements, 1)
AND      WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, PlotSize, 1)
AND      WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, PlinthArea, 1)
AND      WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, SizeOfKitchen, 1)
AND      WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, AreaOfBed1, 1)
AND      WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, AreaOfBed2, 1)
AND      FRAME(UserRequirements, NumberOfBeds, 1) == 2
THEN     WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, Rating, 2)
AND      case information data file ready

#
RULE     writing case information to data file 2
IF       WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, AreaOfBed3, 1)
AND      FRAME(UserRequirements, NumberOfBeds, 1) == 3
THEN     WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, Rating, 2)
AND      case information data file ready

#
RULE     writing case information to data file 3
IF       WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, AreaOfBed4, 1)
AND      FRAME(UserRequirements, NumberOfBeds, 1) == 4
THEN     WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, Rating, 2)
AND      case information data file ready

#
RULE     writing case information to data file 4
IF       WRITE cinfo.dat NUMERIC: 2, FRAME(UserRequirements, AreaOfBed5, 1)
THEN     WRITE cinfo.dat NUMERIC: 2, FRAME(Residence, Rating, 2)
AND      case information data file ready

#
RULE     storing case info
IF       case card created
AND      case information data file ready
AND      CBR_add_case(case info data file) == 1
AND      CBR_save_case_card(case index) == 1
AND      CBR_unload_case_card(case index) == 1
THEN     case information stored

#
RULE     for bedroom instances
IF       FADD_ATTR Residence, FSTRING, BedRoom
AND      FINSERT_VAL Residence, BedRoom, 1, BedRoom
AND      FCREATE_FRAME BedRoom
AND      FADD_ATTR BedRoom, FDOUBLE, Length
AND      FADD_ATTR BedRoom, FDOUBLE, Breadth
AND      FADD_ATTR BedRoom, FSTRING, Bath
AND      FADD_ATTR BedRoom, FDOUBLE, LeftBottomX
AND      FADD_ATTR BedRoom, FDOUBLE, LeftBottomY
AND      create_instances(parent frame, frame 1, number of beds) == 1
THEN     bedroom instances created

#
RULE     for bedroom instance name
IF       loop index 1 <= number of beds
THEN     bedroom instance IS BedRoom
AND      bedroom instance IS inst_name(bedroom instance, loop index 1)
AND      ASK bed room length
AND      ASK bed room breadth
AND      ASK bath
AND      ASK bed lbx
AND      ASK bed lby

#
RULE     for attached bathroom instance name
IF       loop index 1 <= number of beds
THEN     attached bathroom instance IS AttachedBathRoom
AND      attached bathroom instance IS inst_name(attached bathroom instance, loop index 1)
AND      ASK attached bath length
AND      ASK attached bath breadth
AND      ASK attached bath lbx
AND      ASK attached bath lby

#
RULE     for bath details 1
IF       bath IS attached
AND      FADD_ATTR bedroom instance, FSTRING, bathroom instance
AND      FINSERT_VAL bedroom instance, attached bathroom instance, 1, attached bathroom instance
AND      FCREATE_FRAME attached bathroom instance
AND      FADD_ATTR attached bathroom instance, FDOUBLE, Length
AND      FADD_ATTR attached bathroom instance, FDOUBLE, Breadth
AND      FADD_ATTR attached bathroom instance, FDOUBLE, LeftBottomX
AND      FADD_ATTR attached bathroom instance, FDOUBLE, LeftBottomY
AND      FINSERT_VAL attached bathroom instance, Length, 1, attached bath length
AND      FINSERT_VAL attached bathroom instance, Breadth, 1, attached bath breadth
AND      FINSERT_VAL attached bathroom instance, LeftBottomX, 1, attached bath lbx
AND      FINSERT_VAL attached bathroom instance, LeftBottomY, 1, attached bath lby
THEN     bathroom details read

#
RULE     for bath details 2
IF       bath IS none
THEN     bathroom details read

#
RULE     for reading bedroom dimensions
IF       loop index 1 <= number of beds
AND      DISPLAY bedroom number
AND      FINSERT_VAL bedroom instance, Length, 1, bed room length
AND      FINSERT_VAL bedroom instance, Breadth, 1, bed room breadth
AND      FINSERT_VAL bedroom instance, Bath, 1, bath
AND      FINSERT_VAL bedroom instance, LeftBottomX, 1, bed lbx
AND      FINSERT_VAL bedroom instance, LeftBottomY, 1, bed lby
AND      bathroom details read
THEN     bedroom dimensions are read

#
RULE     for bedroom details
IF       bedroom instances created
AND      bedroom dimensions are read
THEN     bedroom details read

#
RULE     for independent bathroom instance name
IF       loop index 2 <= number of baths
AND      FRAME(UserRequirements, NoOfBaths, 1) > 1
THEN     independent bathroom instance IS BathRoom
AND      independent bathroom instance IS inst_name(independent bathroom instance, loop index 2)
AND      ASK independent bath length
AND      ASK independent bath breadth
AND      ASK independent bath lbx
AND      ASK independent bath lby
ELSE     independent bathroom instance IS BathRoom
AND      ASK independent bath length
AND      ASK independent bath breadth
AND      ASK independent bath lbx
AND      ASK independent bath lby

#
RULE     for independent bathroom instances
IF       FADD_ATTR Residence, FSTRING, BathRoom
AND      FINSERT_VAL Residence, BathRoom, 1, BathRoom
AND      FCREATE_FRAME BathRoom
AND      FADD_ATTR BathRoom, FDOUBLE, Length
AND      FADD_ATTR BathRoom, FDOUBLE, Breadth
AND      FADD_ATTR BathRoom, FDOUBLE, LeftBottomX
AND      FADD_ATTR BathRoom, FDOUBLE, LeftBottomY
AND      FRAME(UserRequirements, NoOfBaths, 1) > 1
AND      create_instances(parent frame, frame 2, number of baths) == 1
THEN     independent bathroom instances created
ELSE     independent bathroom instances created

#
RULE     for reading independent bathroom details
IF       DISPLAY independent bathroom number
AND      FINSERT_VAL independent bathroom instance, Length, 1, independent bath length
AND      FINSERT_VAL independent bathroom instance, Breadth, 1, independent bath breadth
AND      FINSERT_VAL independent bathroom instance, LeftBottomX, 1, independent bath lbx
AND      FINSERT_VAL independent bathroom instance, LeftBottomY, 1, independent bath lby
THEN     independent bathroom details read

#
RULE     for separate bath rooms
IF       FRAME(UserRequirements, Bath, 1) IS yes
AND      loop index 2 <= number of baths
AND      independent bathroom instances created
AND      independent bathroom details read
THEN     separate bathrooms details read
ELSE     separate bathrooms details read

#
RULE     for living dining room
IF       FEXIST_POS UserRequirements, LivingDiningRequirements, 1
THEN     living dining room IS FRAME(UserRequirements, LivingDiningRequirements, 1)

#
RULE     for veandah info
IF       FRAME(UserRequirements, Verandah, 1) IS yes
AND      FADD_ATTR Verandah, FDOUBLE, LeftBottomX
AND      FADD_ATTR Verandah, FDOUBLE, LeftBottomY
AND      FINSERT_VAL Verandah, LeftBottomX, 1, verandah lbx
AND      FINSERT_VAL Verandah, LeftBottomY, 1, verandah lby
THEN     verandah information read
ELSE     verandah information read

#
RULE     reading solution details
IF       bedroom details read
AND      separate bathrooms details read
AND      FADD_ATTR Residence, FDOUBLE, LeftBottomX
AND      FADD_ATTR Residence, FDOUBLE, LeftBottomY
AND      FADD_ATTR Residence, FSTRING, KitchenRoom
AND      FADD_ATTR Residence, FSTRING, living dining room
AND      FINSERT_VAL Residence, LeftBottomX, 1, building lbx
AND      FINSERT_VAL Residence, LeftBottomY, 1, building lby
AND      FINSERT_VAL Residence, KitchenRoom, 1, KitchenRoom
AND      FINSERT_VAL Residence, living dining room, 1, living dining room
# kitchen room
AND      FCREATE_FRAME KitchenRoom
AND      FADD_ATTR KitchenRoom, FDOUBLE, Length
AND      FADD_ATTR KitchenRoom, FDOUBLE, Breadth
AND      FADD_ATTR KitchenRoom, FDOUBLE, LeftBottomX
AND      FADD_ATTR KitchenRoom, FDOUBLE, LeftBottomY
AND      FINSERT_VAL KitchenRoom, Length, 1, kitchen length
AND      FINSERT_VAL KitchenRoom, Breadth, 1, kitchen breadth
AND      FINSERT_VAL KitchenRoom, LeftBottomX, 1, kitchen lbx
AND      FINSERT_VAL KitchenRoom, LeftBottomY, 1, kitchen lby
# living dining room
AND      FCREATE_FRAME living dining room
AND      FADD_ATTR living dining room, FDOUBLE, Length
AND      FADD_ATTR living dining room, FDOUBLE, Breadth
AND      FADD_ATTR living dining room, FDOUBLE, LeftBottomX
AND      FADD_ATTR living dining room, FDOUBLE, LeftBottomY
AND      FINSERT_VAL living dining room, Length, 1, length
AND      FINSERT_VAL living dining room, Breadth, 1, breadth
AND      FINSERT_VAL living dining room, LeftBottomX, 1, lbx
AND      FINSERT_VAL living dining room, LeftBottomY, 1, lby
# verandah
AND      verandah information read
THEN     solution details read

#
RULE     goal rule 1
IF       auto save mode IS_DEFINED
AND      case information stored
AND      FSAVE_FBASE solution name
AND      FDEL_FBASE solution name
THEN     new solution is stored
AND      LOAD control

#
RULE     goal rule 2
IF   NOT auto save mode IS_DEFINED
AND      case information stored
AND      solution details read
AND      FSAVE_FBASE solution name
AND      FDEL_FBASE solution name
THEN     new solution is stored
AND      LOAD control

# CONSTRAINTS

RULE C1
IF   bedroom dimensions are read
AND  loop index 1 < number of beds
THEN loop index 1 = loop index 1 + 1

RULE C2
IF   separate bathrooms details read
AND  loop index 2 < number of baths
THEN loop index 2 = loop index 2 + 1

$INCLUDE storer.inf

END
