
/* MYLAYCRIT.C */

#include <stdio.h>
#ifdef __TURBOC__
#include <alloc.h>
#else
#include <malloc.h>
#endif
#include "xfms.h"

/* Don't change the below definitions and declarations */

#define  INT   4
#define  DOU   5
#define  STR   6

extern union
{
  int Int;
  double Dou;
  char Str[500];
} GENCRIT_proc_value;
extern GENCRIT_unload_args();
typedef int (*gencrit_funcptr_type)();
struct GENCRIT_userfn
{
  char *name;
  gencrit_funcptr_type func;
};
extern int GENCRIT_unload_args();

/* function declarations (user defined) */

/* functions that are to be linked with GENCRIT KB */
int determine_no_of_spacings ();
int determine_beam_to_beam_compatibility_number ();
int determine_beam_to_column_lng_compatibility_number ();
int determine_beam_to_column_trn_compatibility_number ();
int determine_interference_in_same_direction();
int determine_interference_in_across_direction();

/* other functions */


/* you can include your functions here like this */


struct GENCRIT_userfn GENCRIT_userfn_Rec[]=
{
/*
  {"<as refered in GENCRIT KB>", <as declared here>},
*/
  {"DetermineNoOfSpacings",determine_no_of_spacings},
  {"DetermineBmtoBmCompatibilityNumber", \
		determine_beam_to_beam_compatibility_number},
  {"DetermineBmtoCmLngCompatibilityNumber", \
		determine_beam_to_column_lng_compatibility_number},
  {"DetermineBmtoCmTrnCompatibilityNumber", \
		determine_beam_to_column_trn_compatibility_number},
  {"DetermineInterferenceInSameDirection", \
		determine_interference_in_same_direction},
  {"DetermineInterferenceInAcrossDirection", \
		determine_interference_in_across_direction},
  {"END_MARKER",0}, /* this line is a must */
};

/* Usually a c function is written as shown below. */

/* Function to calculate the number of different bay spacings in a
   particular layout */

int determine_no_of_spacings()
{
  int no_bays,n_spacings=0,i,j ;
  double *spacing,rating ;
  void *val ;
  char frame[10] ;

  val     = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("lng_layout","no_bays",1,val) ;
  no_bays = *(int *)val;

  spacing = (double *)malloc(no_bays*sizeof(double));


  for (i=0; i<no_bays; i++)
  {
    sprintf(frame,"bay[%d]",i+1);
    fms_get_val(frame,"spacing",1,val) ;
    spacing[i] = *(double *)val;
  }

  for(i=0;i<no_bays;i++)
  {
    int already_exist = 0;
    for (j=0; j<i; j++)
    {
      if(spacing[i] == spacing[j])
      {
	already_exist = 1;
	break;
      }
    }
    if(!already_exist)
    n_spacings++;
  }

  rating=(double)(no_bays-n_spacings)/(no_bays-1)*100 ;
  GENCRIT_proc_value.Dou=rating;
  return(DOU);
}
/* Function to find the beam to beam compatibility number*/

int determine_beam_to_beam_compatibility_number()
{
  int no_beams,no_jns=0,comp_no=0,i,j,found=0,ch_no ;
  double rating,*depth ;
  void *val ;
  char frame[10],orientation[5],f_orient[5] ;

  val = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("bldg_system","no_beams",1,val) ;
  no_beams = *(int *)val ;

  depth = (double *)malloc(no_beams*sizeof(double)) ;


  for (i=0; i<no_beams; i++)
  {
    sprintf(frame,"beam[%d]",i+1) ;
    fms_get_val(frame,"depth",1,val) ;
    depth[i] = *(double *)val ;

    fms_get_val(frame,"orientation",1,val) ;
    strcpy (orientation,(char *)val) ;

    if (i == 0)
    strcpy(f_orient,val) ;
    if (!found && strcmp(f_orient,orientation)!=0)
    {
      found=1 ;
      ch_no=i ;
    }
  }

  for(i=0;i<ch_no;i++)
  {
    for (j=ch_no; j<no_beams; j++)
    {
      no_jns++ ;
      if(depth[i] != depth[j])
      comp_no++ ;
    }
  }

  rating=(double)(no_jns-comp_no)/(no_jns)*100 ;
  GENCRIT_proc_value.Dou=rating ;
  return(DOU) ;
}


int determine_beam_to_column_lng_compatibility_number()
{
  void *val ;
  int i, no_columns, comp_no=0 ;
  char frame[25], frame1[25] ;
  double width, dim_trn, rating ;

  val     = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("bldg_system","no_columns",1,val) ;
  no_columns = *(int *)val ;

  for (i=0; i<no_columns; i++)
  {
    sprintf(frame,"column[%d]",i+1) ;
    fms_get_val(frame,"adj_lng_beam",1,val) ;
    strcpy (frame1,(char *)val) ;

    fms_get_val(frame1,"width",1,val) ;
    width = *(double *)val ;
    fms_get_val(frame,"dim_trn",1,val) ;
    dim_trn = *(double *)val ;


    if (width != dim_trn)
    comp_no++ ;
  }

  rating=(double)(no_columns-comp_no)/(no_columns)*100 ;
  GENCRIT_proc_value.Dou=rating ;
  return(DOU) ;
}

int determine_beam_to_column_trn_compatibility_number()
{
  void *val ;
  int i, no_columns, comp_no=0 ;
  char frame[25], frame1[25] ;
  double dim_lng, width, rating ;

  val     = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("bldg_system","no_columns",1,val) ;
  no_columns = *(int *)val ;

  for (i=0; i<no_columns; i++)
  {
    sprintf(frame,"column[%d]",i+1) ;
    fms_get_val(frame,"adj_trn_beam",1,val) ;
    strcpy (frame1,(char *)val) ;

    fms_get_val(frame1,"width",1,val) ;
    width = *(double *)val ;
    fms_get_val(frame,"dim_lng",1,val) ;
    dim_lng = *(double *)val ;


    if (width != dim_lng)
    comp_no++ ;
  }

  rating=(double)(no_columns-comp_no)/(no_columns)*100 ;
  GENCRIT_proc_value.Dou=rating ;
  return(DOU) ;
}

int determine_interference_in_same_direction()
{
  void *val ;
  int i, j, no_pipes, no_beams, intr_no=0 ;
  char drn_p[10], drn_b[10], frame[10], frame1[10] ;
  double dia, pos_b, pos_p, pos_b1, pos_b2, flr_height ;
  double width, depth, height, rating, z_coord, saf_height ;

  val     = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("bldg_system","no_pipes",1,val) ;
  no_pipes = *(int *)val ;

  fms_get_val("bldg_system","flr_height",1,val) ;
  flr_height = *(double *)val ;

  fms_get_val("bldg_system","no_beams",1,val) ;
  no_beams = *(int *)val ;


  for (i=0; i<no_pipes; i++)
  {
    sprintf(frame,"pipe[%d]",i+1) ;
    fms_get_val(frame,"z_coord",1,val) ;
    z_coord = *(double *)val ;

    fms_get_val(frame,"position",1,val) ;
    pos_p = *(double *)val ;

    fms_get_val(frame,"dia",1,val) ;
    dia = *(double *)val ;

    height = z_coord+dia/2 ;

    fms_get_val(frame,"orientation",1,val) ;
    strcpy (drn_p,(char *)val) ;

    for (j=0; j<no_beams; j++)
    {
      sprintf(frame1,"beam[%d]",j+1) ;
      fms_get_val(frame1,"orientation",1,val) ;
      strcpy (drn_b,(char *)val) ;
      if (strcmp(drn_p,drn_b)==0)
      {
	fms_get_val(frame1,"position",1,val) ;
	pos_b = *(double *)val ;
	fms_get_val(frame1,"width",1,val) ;
	width = *(double *)val ;
	fms_get_val(frame1,"depth",1,val) ;
	depth = *(double *)val ;

	saf_height = flr_height - depth ;
	pos_b1 = pos_b+width/2+dia ;
	pos_b2 = pos_b-width/2-dia ;

	if(pos_b2<=pos_p && pos_p<=pos_b1 && height>saf_height)
	intr_no++ ;
      }
    }
  }

  rating=(double)(no_pipes-intr_no)/(no_pipes)*100 ;
  GENCRIT_proc_value.Dou=rating ;
  return(DOU) ;
}

int determine_interference_in_across_direction()
{
  void *val ;
  int i, j, no_pipes, no_beams, intr_no=0, no_jns=0 ;
  char drn_p[10], drn_b[10], frame[10], frame1[10] ;
  double dia, pos_b, pos_p, flr_height ;
  double width, depth, height, rating, z_coord, saf_height ;

  val     = (void *)malloc(100*sizeof(char)) ;

  fms_get_val("bldg_system","no_pipes",1,val) ;
  no_pipes = *(int *)val ;

  fms_get_val("bldg_system","flr_height",1,val) ;
  flr_height = *(double *)val ;

  fms_get_val("bldg_system","no_beams",1,val) ;
  no_beams = *(int *)val ;


  for (i=0; i<no_pipes; i++)
  {
    sprintf(frame,"pipe[%d]",i+1) ;
    fms_get_val(frame,"z_coord",1,val) ;
    z_coord = *(double *)val ;

    fms_get_val(frame,"position",1,val) ;
    pos_p = *(double *)val ;

    fms_get_val(frame,"dia",1,val) ;
    dia = *(double *)val ;

    height = z_coord+dia/2 ;

    fms_get_val(frame,"orientation",1,val) ;
    strcpy (drn_p,(char *)val) ;

    for (j=0; j<no_beams; j++)
    {
      sprintf(frame1,"beam[%d]",j+1) ;
      fms_get_val(frame1,"orientation",1,val) ;
      strcpy (drn_b,(char *)val) ;
      if (strcmp(drn_p,drn_b)!=0)
      {
	fms_get_val(frame1,"width",1,val) ;
	width = *(double *)val ;
	fms_get_val(frame1,"depth",1,val) ;
	depth = *(double *)val ;
	saf_height = flr_height - depth ;

	if( height > saf_height )
	intr_no++ ;

	no_jns++ ;
      }
    }
  }
  rating=(double)(no_jns-intr_no)/(no_jns)*100 ;
  GENCRIT_proc_value.Dou=rating ;
  return(DOU) ;
}
